#include "LineasTransmision.h"

extern PegBitmap gbltBitmap;

LineasTransmision::LineasTransmision(PegRect rect, CPMainFrame* frame):CPModuleWindow(rect,0,0,frame)
{
init();
}

int LineasTransmision::init()
{   
   	const WORD wStyle = FF_NONE|TJ_LEFT|AF_TRANSPARENT| TT_COPY;
	const SIGNED margin = 10;
	const SIGNED width=mClient.Width()-2*margin;

    PEGCHAR* chrohm = "\xEC\x7E"; 

    PegRect rectangle;
 
 	PegPrompt* label_d = new PegPrompt(margin, 65, width, "d(cm):",0,wStyle);
	d = new CPPegString(margin+45,65,width - 70,"",0, FF_THIN|AF_ENABLED|EF_EDIT);
	AddR(d);
	AddR(label_d);
	
 	PegPrompt* label_f = new PegPrompt(margin, 80, width, "f(Mhz):",0,wStyle);
	f = new CPPegString(margin+45,80,width - 70,"",0, FF_THIN|AF_ENABLED|EF_EDIT);
	AddR(f);
	AddR(label_f);
	
 	PegPrompt* label_Zo = new PegPrompt(margin, 95, width, "Zo("+CPString(chrohm)+"):",0,wStyle);
	Zo = new CPPegString(margin+45,95,width - 70,"",0, FF_THIN|AF_ENABLED|EF_EDIT);
	AddR(Zo);
	AddR(label_Zo);
    	
 	PegPrompt* label_Rc = new PegPrompt(margin, 110, width/2, "Rc:",0,wStyle);
	Rc = new CPPegString(margin+20,110,40,"",0, FF_THIN|AF_ENABLED|EF_EDIT);
	AddR(Rc);
	AddR(label_Rc);
    
 	PegPrompt* label_Xc = new PegPrompt(margin+width/2, 110, width/2, "Xc:",0,wStyle);
	Xc = new CPPegString(97,110,40,"",0, FF_THIN|AF_ENABLED|EF_EDIT);
	AddR(Xc);
	AddR(label_Xc);
	
    //rectangle.Set(MAINFRAME_LEFT, MAINFRAME_TOP, MAINFRAME_RIGHT, MAINFRAME_BOTTOM);
    rectangle.Set(35, 130, 100, 145);
   	PegTextButton* b3 = new PegTextButton(rectangle, "Calcular!", BOTO, AF_ENABLED|TJ_CENTER);
	AddR(b3);
	
    rectangle.Set(MAINFRAME_LEFT, MAINFRAME_TOP, MAINFRAME_RIGHT, MAINFRAME_TOP+65);
 	PegBitmapButton * imatge = new PegBitmapButton(rectangle,&gbltBitmap,0,AF_TRANSPARENT,TYPE_PROMPT,FALSE);
 	AddR(imatge); 
 	
 	rectangle.Set(10, 150, MAINFRAME_RIGHT-10, 185); 
    cList = new PegVertList(rectangle,ID_RESULTS);
    cList->SetScrollMode(WSM_AUTOVSCROLL); 
    AddR(cList);
    
	return 0;
}

void LineasTransmision::Draw()
{
	BeginDraw();
	Invalidate(mClient);
	DrawFrame();
	DrawChildren();
	EndDraw();
}

SIGNED LineasTransmision::Message(const PegMessage &Mesg)
{
	switch(Mesg.wType)
	{
		case SIGNAL(BOTO, PSF_CLICKED):
		{
             Calcula();
		}
		break;
		
		default:
		return CPModuleWindow::Message(Mesg);
	}
	return 0;
}

WORD LineasTransmision::Dialog(char* title, char* message)
{
	PegRect rr = GetMainFrame()->TopAppRectangle();
	rr.wBottom -= 30;
	rr.wRight -= 10;
	rr.wTop += 10;
	rr.wLeft += 10;
	PegMessageWindow *msg = new PegMessageWindow(rr, title, message, MW_OK, 0,0,0);
	return msg->Execute();
}

void LineasTransmision::Calcula()
{
    CBCD Zc, Ze, caux, caux2, caux3, gamma_e, gamma_c;
    OBCD aux, aux2, beta, lambda, freq, l, Z_o;
    CP_CHAR buffer[30];
    
    // Carceres especiales
    PEGCHAR* chrohm = "\xEC\x7E"; 
    PEGCHAR* chrbeta = "\xED\x68"; 
    PEGCHAR* chrgamma = "\xEC\x69"; 
    PEGCHAR* chrlambda = "\xED\x71"; 
    
    // Borrar la lista de resultados
    cList->Clear();
    
    // Coger los parmetros del usuario
    freq = CPExpression(f->DataGet());
    l = CPExpression(d->DataGet());
    Z_o = CPExpression(Zo->DataGet());

    // La impedancia de carga es compleja, por lo que se carga
    // el valor real y el imaginario por separado
    aux = CPExpression(Rc->DataGet());
    Zc.repart = aux;
    aux = CPExpression(Xc->DataGet());
    Zc.impart = aux;
    
    // Para una velocidad de la onda mxima (la de la luz) poner 300
    // Para otras velocidades ms reales dividir 300 entre la raz de la permitividad relativa
    // o poner la velocidad medida dividida por 10^6 (la frecuencia est en MHz)
    ExecuteToOBCD("300",lambda); 
    Cal_div_OBC(&lambda,&freq);

    aux = lambda;
    Cal_cintx_OBC(&aux,4);
 
    // Poner la lambda encontrada en la lista de resultados
    cList->AddToEnd(new PegPrompt(0, 0, CPString(chrlambda)+" = "+CPString(aux)+" m", 10, FF_NONE|TJ_LEFT|AF_ENABLED|TT_COPY));
    
    // El parmetro beta=2*Pi/lambda
    Cal_set2pi_OBC(&beta);
    Cal_div_OBC(&beta,&lambda);

    aux = beta;
    Cal_cintx_OBC(&aux,4);

    // Poner la beta encontrada en la lista de resultados
    cList->AddToEnd(new PegPrompt(0, 0, CPString(chrbeta)+" = "+CPString(aux)+" rad/m", 10, FF_NONE|TJ_LEFT|AF_ENABLED|TT_COPY));
    
    // Convertir el valor imaginario en un array
    CP_15digit_CMP(&Zc, buffer);
      
    cList->AddToEnd(new PegPrompt(0, 0, "Zc = "+CPString((PEGCHAR*)buffer)+" "+CPString(chrohm), 10, FF_NONE|TJ_LEFT|AF_ENABLED|TT_COPY));

    // Buscar la admitancia de carga, invirtiendo la impedancia de carga encontrada
    Cal_setn_OBC(1,&caux.repart);
    Cal_setn_OBC(0,&caux.impart);
    Cal_div_CMP(&caux,&Zc);

    // Redondear parte real e imaginaria del valor de admitancia encontrado
    Cal_cintx_OBC(&caux.repart,4);
    Cal_cintx_OBC(&caux.impart,4);
    CP_15digit_CMP(&caux, buffer);
      
    cList->AddToEnd(new PegPrompt(0, 0, "Yc = "+CPString((PEGCHAR*)buffer)+" S", 10, FF_NONE|TJ_LEFT|AF_ENABLED|TT_COPY));
    
    ExecuteToOBCD("0.01",aux);
    Cal_mul_OBC(&aux,&l);
    Cal_mul_OBC(&aux,&beta);
    Cal_tan_OBC(&aux,IM_CAL_ANGL_RAD); // aux = tg ( d(cm) / 100 * beta )
    
    Cal_seti(&caux);
    Cal_mul_OBC(&caux.impart,&aux);
    Cal_mul_OBC(&caux.impart,&Z_o);
    Cal_add_CMP(&caux,&Zc); // caux = Zc + j * Zo * tg(beta*d)
    
    Cal_seti(&caux2);
    Cal_mul_OBC(&caux2.impart,&aux);
    Cal_mul_CMP(&caux2,&Zc);
    Cal_add_OBC(&caux2.repart,&Z_o); // caux2 = Zo + j*Zc*tg(beta*d)
    
    Cal_div_CMP(&caux,&caux2); // caux = ( Zc + j * Zo * tg(beta*d)) / (Zo + j*Zc*tg(beta*d))

    caux2.repart = Z_o;
    Cal_setn_OBC(0,&caux2.impart);    
    Cal_mul_CMP(&caux,&caux2); // caux = Zo * ( Zc + j * Zo * tg(beta*d)) / (Zo + j*Zc*tg(beta*d))
    
    Ze = caux;
    
    Cal_cintx_OBC(&caux.repart,4);
    Cal_cintx_OBC(&caux.impart,4);
    CP_15digit_CMP(&caux, buffer);

    cList->AddToEnd(new PegPrompt(0, 0, "Ze = "+CPString((PEGCHAR*)buffer)+" "+CPString(chrohm), 10, FF_NONE|TJ_LEFT|AF_ENABLED|TT_COPY));

    Cal_setn_OBC(1,&caux.repart);
    Cal_setn_OBC(0,&caux.impart);
    Cal_div_CMP(&caux,&Ze);

    Cal_cintx_OBC(&caux.repart,4);
    Cal_cintx_OBC(&caux.impart,4);
    CP_15digit_CMP(&caux, buffer);

    cList->AddToEnd(new PegPrompt(0, 0, "Ye = "+CPString((PEGCHAR*)buffer)+" S", 10, FF_NONE|TJ_LEFT|AF_ENABLED|TT_COPY));
    
    caux = Zc;
    Cal_add_CMP(&caux,&caux2);
    caux3 = Zc;
    Cal_sub_CMP(&caux3,&caux2);
    Cal_div_CMP(&caux3,&caux);

    gamma_c = caux3;
    
    Cal_cintx_OBC(&caux3.repart,4);
    Cal_cintx_OBC(&caux3.impart,4);
    CP_15digit_CMP(&caux3, buffer);
    
    cList->AddToEnd(new PegPrompt(0, 0, CPString(chrgamma)+"c = "+CPString((PEGCHAR*)buffer), 10, FF_NONE|TJ_LEFT|AF_ENABLED|TT_COPY));

    caux = Ze;
    Cal_add_CMP(&caux,&caux2);
    caux3 = Ze;
    Cal_sub_CMP(&caux3,&caux2);
    Cal_div_CMP(&caux3,&caux);

    gamma_e = caux3;
    
    Cal_cintx_OBC(&caux3.repart,4);
    Cal_cintx_OBC(&caux3.impart,4);
    CP_15digit_CMP(&caux3, buffer);
    
    cList->AddToEnd(new PegPrompt(0, 0, CPString(chrgamma)+"e = "+CPString((PEGCHAR*)buffer), 10, FF_NONE|TJ_LEFT|AF_ENABLED|TT_COPY));

    // Calcular el mdulo de los coeficientes de reflexin,
    // que nos permitirn encontrar la relacin de onda estacionaria (SWR)
    Cal_abs_CMP(&gamma_e);
    Cal_abs_CMP(&gamma_c);    

    Cal_setn_OBC(1,&aux);
    Cal_sub_OBC(&aux,&gamma_c.repart);
    Cal_setn_OBC(1,&aux2);
    Cal_add_OBC(&aux2,&gamma_c.repart);
    Cal_div_OBC(&aux2,&aux);
    Cal_cintx_OBC(&aux2,4);
    
    cList->AddToEnd(new PegPrompt(0, 0, "SWRc= "+CPString(aux2), 10, FF_NONE|TJ_LEFT|AF_ENABLED|TT_COPY));
    
    Cal_setn_OBC(1,&aux);
    Cal_sub_OBC(&aux,&gamma_e.repart);
    Cal_setn_OBC(1,&aux2);
    Cal_add_OBC(&aux2,&gamma_e.repart);
    Cal_div_OBC(&aux2,&aux);
    Cal_cintx_OBC(&aux2,4);

    cList->AddToEnd(new PegPrompt(0, 0, "SWRe= "+CPString(aux2), 10, FF_NONE|TJ_LEFT|AF_ENABLED|TT_COPY));
    
    Draw();
}   

extern "C" char *ExtensionGetLang(ID_MESSAGE MessageNo)
{
return "";
}
